//
//  Cocoa_SR_Test.m
//  Cocoa_SR_Test
//
//  Created by Nao Tokui on 06/04/21.
//  Copyright 2006 Nao Tokui. All rights reserved.
//

#import "Cocoa_SR_Test.h"

#define OBSERVED_OBJECT_STRING @"com.naotokui.maxmsp" 

// These strings must be shared with the Max Object. 

#define SHARED_DICTIONARY_KEY_FOR_INT @"MAX_COCOA_INT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_FLOAT @"MAX_COCOA_FLOAT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_ANYTHING @"MAX_COCOA_ANYTHING_KEY"

#pragma mark SET DEFAULT CHANNEL NAME

// Default Channel Name
#define CH_NAME_FROM_COCOA		@"FROM_Cocoa"
#define CH_NAME_TO_COCOA		@"TO_Cocoa"


@implementation Cocoa_SR_Test

- (void) awakeFromNib
{
	/* Set Default Channel Name */
	[self setSendChName: CH_NAME_FROM_COCOA ];
	[self setReceiveChName: CH_NAME_TO_COCOA ];
}

- (void) dealloc {
	[sendChName release];
	[receiveChName release];
	[super dealloc];
}


- (void) setSendChName: (NSString *) name {
	[self willChangeValueForKey:@"sendChName"];
	sendChName = [name retain]; 
	[self didChangeValueForKey:@"sendChName"];
}

- (NSString *) sendChName { return sendChName; }

- (void) setReceiveChName: (NSString *) name {
	NSString *oldChName = receiveChName;
	[self willChangeValueForKey:@"receiveChName"];
	receiveChName = [name retain]; 
	[self didChangeValueForKey:@"receiveChName"];

	/* Registering to Receive Notification */
	NSString *observedObject = OBSERVED_OBJECT_STRING;
	NSDistributedNotificationCenter *center = [NSDistributedNotificationCenter defaultCenter];
	[center removeObserver:self  name:oldChName object:observedObject];			// remove old name
	[center addObserver: self selector: @selector(callbackWithNotification:)	// register with new name
            name: receiveChName object: observedObject];

	[oldChName release];
}
- (NSString *) receiveChName { return receiveChName; }

- (IBAction) setSliderValue: (id) sender
{	
	[self sendFloat: [sender floatValue]];
}

- (IBAction) setTextFieldValue: (id) sender
{
	[self sendString: [sender stringValue]];
}


#pragma mark
#pragma mark SEMD TO Max/MSP 

- (void) sendString: (NSString *) str
{
	[self sendNotification: str withKey:  SHARED_DICTIONARY_KEY_FOR_ANYTHING];
}

- (void) sendFloat: (float) number
{
	[self sendNotification: [NSNumber numberWithFloat: number] withKey:  SHARED_DICTIONARY_KEY_FOR_FLOAT];
}

- (void) sendInteger: (int) number
{
	[self sendNotification: [NSNumber numberWithInt: number] withKey:  SHARED_DICTIONARY_KEY_FOR_INT];
}

- (void) sendNotification: (id) value withKey: (NSString *) key 
{
	NSDistributedNotificationCenter *center = [NSDistributedNotificationCenter defaultCenter];
	
	NSDictionary *info = [NSDictionary dictionaryWithObject: value forKey: key];
	
	[center postNotificationName: sendChName /* Specified Channel Name is used here. */
            object:  OBSERVED_OBJECT_STRING /* Observed object must be a string in the case of distributed notification*/
            userInfo: info /* no dictionary */
            deliverImmediately: YES];
}

#pragma mark
#pragma mark RECEIVE FROM Max/MSP 

// CUSTOMIZE THIS CALLBACK

- (void) callbackWithNotification:(NSNotification *)myNotification
{
	NSDictionary *dict = [myNotification userInfo];
	// a string value received
	NSString *string = [dict valueForKey: SHARED_DICTIONARY_KEY_FOR_ANYTHING] ;
	if (string) [textField setStringValue:string];

	// an int value received	
	NSNumber *intnum = [dict valueForKey: SHARED_DICTIONARY_KEY_FOR_INT] ;
	if (intnum) [slider setIntValue: [intnum intValue]];	

	// a float value received
	NSNumber *floatnum = [dict valueForKey: SHARED_DICTIONARY_KEY_FOR_FLOAT] ;
	if (floatnum) [slider setFloatValue: [floatnum floatValue]];
}

@end
