#include "ext.h"

/* ---- cocoa_receive.c --- 
	Simple communication with other Cocoa/Carbon Apps
	Nao Tokui 2006 www.naotokui.com
-- */

typedef struct {
	t_object c_ob;
	void *c_outlet;
} t_cocoa_receive;

#define OBSERVED_OBJECT_STRING "com.naotokui.maxmsp"

#define SHARED_DICTIONARY_KEY_FOR_INT "MAX_COCOA_INT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_FLOAT "MAX_COCOA_FLOAT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_ANYTHING "MAX_COCOA_ANYTHING_KEY"

#define DBLCLICK_MESSAGE "cocoa_receive v0.55b : nao tokui 2006 : www.naotokui.com"

void cocoa_receive_set(t_cocoa_receive *x, t_symbol *s);
void cocoa_receive_dblclick(t_cocoa_receive *x);
void cocoa_receive_assist(t_cocoa_receive *x, void *b, long m, long a, char *s);
void *cocoa_receive_new(t_symbol *s);

void cocoa_receive_notification_received(CFNotificationCenterRef center, void *observer,
            CFStringRef notificationName, const void *observedObject,
            CFDictionaryRef userInfo);

void *cocoa_receive_class;
t_symbol *ps_list;

void main()
{
	setup((t_messlist **)&cocoa_receive_class, (method)cocoa_receive_new, 0L, (short)sizeof(t_cocoa_receive), 0L, A_SYM, 0);
	addmess((method)cocoa_receive_set, "set", A_SYM, 0);
	addmess((method)cocoa_receive_assist,"assist",A_CANT,0);
	addmess((method)cocoa_receive_dblclick,"dblclick",A_CANT,0);
	finder_addclass("System","cocoa_receive");
	ps_list = gensym("list");			
}

void cocoa_receive_set(t_cocoa_receive *x, t_symbol *s)
{	
	CFStringRef str = CFStringCreateWithCString(NULL, s->s_name, kCFStringEncodingMacRoman);

	void *observer = x;
	CFStringRef observedObject = CFSTR(OBSERVED_OBJECT_STRING);
	CFNotificationCenterRef center = CFNotificationCenterGetDistributedCenter();
	CFNotificationCenterAddObserver(center, observer, cocoa_receive_notification_received,
			str, observedObject,
            CFNotificationSuspensionBehaviorDeliverImmediately);	
}

void cocoa_receive_dblclick(t_cocoa_receive *x)
{
	post(DBLCLICK_MESSAGE);
}

void cocoa_receive_assist(t_cocoa_receive *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_INLET)
		sprintf(s,"set channel name");
	else 
		sprintf(s,"Received Message");
}

void cocoa_receive_notification_received(CFNotificationCenterRef center, void *observer,
            CFStringRef notificationName, const void *observedObject,
            CFDictionaryRef userInfo)
{
	t_cocoa_receive *x = observer;

	// a string value received
	CFStringRef str = CFDictionaryGetValue(userInfo, CFSTR(SHARED_DICTIONARY_KEY_FOR_ANYTHING));
	if (str != NULL) {
		CFIndex lng = CFStringGetLength(str);
		char *buffer = malloc(sizeof(char) * (lng+1));
		CFStringGetCString(str, buffer, lng+1, kCFStringEncodingUTF8);

		t_atom atom;
		SETSYM(&atom, gensym(buffer));  
		outlet_anything(x->c_outlet, gensym("symbol"), 1, &atom); // output received string!!
	}
	
	// a int value received
	CFNumberRef intnum = CFDictionaryGetValue(userInfo, CFSTR(SHARED_DICTIONARY_KEY_FOR_INT));
	if (intnum != NULL) {
		int n;
		CFNumberGetValue(intnum, kCFNumberIntType, &n);
		outlet_int(x->c_outlet, n);
	}
	
	// a float value received
	CFNumberRef floatnum = CFDictionaryGetValue(userInfo, CFSTR(SHARED_DICTIONARY_KEY_FOR_FLOAT));
	if (floatnum != NULL) {
		float f;
		CFNumberGetValue(floatnum, kCFNumberFloatType, &f);
		outlet_float(x->c_outlet, f);
	}
}

void *cocoa_receive_new(t_symbol *s)
{
	t_cocoa_receive *x;
	x = (t_cocoa_receive *)newobject(cocoa_receive_class);

	x->c_outlet = outlet_new (x,0L);   // for result output
	cocoa_receive_set(x, s);		 // set channel name for receiving data
	
	return x;
}

