#include "ext.h"

/* ---- cocoa_send.c --- 
	Simple communication with other Cocoa/Carbon Apps
	Nao Tokui 2006 www.naotokui.com
-- */

typedef struct {
	t_object c_ob;
	void *c_outlet;
	t_symbol *c_chsym;
} t_cocoa_send;

#define OBSERVED_OBJECT_STRING "com.naotokui.maxmsp"

#define SHARED_DICTIONARY_KEY_FOR_INT "MAX_COCOA_INT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_FLOAT "MAX_COCOA_FLOAT_KEY"
#define SHARED_DICTIONARY_KEY_FOR_ANYTHING "MAX_COCOA_ANYTHING_KEY"

#define MAX_SIZE 512

#define DBLCLICK_MESSAGE "cocoa_send v0.55b : nao tokui 2006 : www.naotokui.com"


void cocoa_send_set(t_cocoa_send *x, t_symbol *s);
void cocoa_send_int(t_cocoa_send *x, long n);
void cocoa_send_float(t_cocoa_send *x, float f);
void cocoa_send_anything(t_cocoa_send *x,t_symbol *s, short argc, t_atom *argv);
void cocoa_send_dblclick(t_cocoa_send *x);
void cocoa_send_assist(t_cocoa_send *x, void *b, long m, long a, char *s);
void *cocoa_send_new(t_symbol *s);

void cocoa_send_notify(t_cocoa_send *x, CFStringRef key, void *value);
static char * to_text(t_cocoa_send *x,t_symbol *s, short argc, t_atom *argv);

void *cocoa_send_class;
t_symbol *ps_list;

void main()
{
	setup((t_messlist **)&cocoa_send_class, (method)cocoa_send_new, 0L, (short)sizeof(t_cocoa_send), 0L, A_SYM, 0);
	addmess((method)cocoa_send_set, "set", A_SYM, 0);
	addint((method)cocoa_send_int);
	addfloat((method)cocoa_send_float);
	addmess((method)cocoa_send_assist,"assist",A_CANT,0);
	addmess((method)cocoa_send_anything,"anything",A_GIMME,0);
	addmess((method)cocoa_send_dblclick,"dblclick",A_CANT,0);
	finder_addclass("System","cocoa_send");
	ps_list = gensym("list");			
}

void cocoa_send_set(t_cocoa_send *x, t_symbol *s)
{	
	x->c_chsym = s;
}

void cocoa_send_dblclick(t_cocoa_send *x)
{
	post(DBLCLICK_MESSAGE);
}

void cocoa_send_assist(t_cocoa_send *x, void *b, long m, long a, char *s)
{
	if (m == ASSIST_INLET)
		sprintf(s,"messages to send / set channel name");
	else 
		sprintf(s,"bang when done");
}

void cocoa_send_int(t_cocoa_send *x, long n)
{
	CFNumberRef cfNum = CFNumberCreate(NULL, kCFNumberLongType, &n);
	cocoa_send_notify(x, CFSTR(SHARED_DICTIONARY_KEY_FOR_INT), (void *)cfNum);
}

void cocoa_send_float(t_cocoa_send *x, float f)
{
	CFNumberRef cfNum = CFNumberCreate(NULL, kCFNumberFloatType, &f);
	cocoa_send_notify(x, CFSTR(SHARED_DICTIONARY_KEY_FOR_FLOAT), (void *)cfNum);
}

void cocoa_send_anything(t_cocoa_send *x,t_symbol *s, short argc, t_atom *argv)
{
	char *message = to_text(x, s, argc, argv);
	CFStringRef cfMessage = CFStringCreateWithCString(NULL, message, kCFStringEncodingMacRoman);
	cocoa_send_notify(x, CFSTR(SHARED_DICTIONARY_KEY_FOR_ANYTHING), (void *)cfMessage);
}

void cocoa_send_notify(t_cocoa_send *x, CFStringRef key, void *value)
{
	// make dictionary 
	CFMutableDictionaryRef userInfo = CFDictionaryCreateMutable(NULL, (CFIndex)1, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
	CFDictionaryAddValue(userInfo, key, value);
	
	// send notify with the dictionary
	CFStringRef observedObject = CFSTR(OBSERVED_OBJECT_STRING);
	CFStringRef cfChName = CFStringCreateWithCString(NULL, x->c_chsym->s_name, kCFStringEncodingMacRoman);
	CFNotificationCenterRef center = CFNotificationCenterGetDistributedCenter();
	CFNotificationCenterPostNotification(center, cfChName, observedObject, userInfo, TRUE);
}


static char * to_text(t_cocoa_send *x,t_symbol *s, short argc, t_atom *argv)
{
	static char tmp[MAX_SIZE];
    short z,i=0;
	char num[32];
	
	if(s) {
	    strcpy(tmp,s->s_name);
	    i=strlen(s->s_name)+1;
	    strcat(tmp," ");
    }
    
    for(z=0;z<argc;z++)
    {

        switch (argv[z].a_type)
        {
         
            case A_LONG:
				sprintf(num, "%d", (int)argv[z].a_w.w_long);
              	strcpy(tmp+i,num);
				i=strlen(num)+i+1;
				strcat(tmp," ");
                break;
            case A_SYM:
				strcpy((tmp+i),argv[z].a_w.w_sym->s_name);
				i=strlen(argv[z].a_w.w_sym->s_name)+i+1;
				strcat(tmp," ");
                break;
            case A_FLOAT:
				sprintf(num, "%f", argv[z].a_w.w_float);
				strcpy(tmp+i,num);
				i=strlen(num)+i+1;
				strcat(tmp," ");
                break;      
        }         
    }
	return tmp;
}


void *cocoa_send_new(t_symbol *s)
{
	t_cocoa_send *x;
	x = (t_cocoa_send *)newobject(cocoa_send_class);

	x->c_outlet = bangout(x);   // for result output
	x->c_chsym = s;		 // set channel name for sending data
	
	return x;
}

